"""
Copyright 2021-2024 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import re

from packaging import version
from packaging.version import Version, VERSION_PATTERN

from Setter.Common.Command import Command
from Setter.Enums.Software import SoftwareCommand
from Setter.Exceptions import CommandException, VersionParseException


class VersionParser:
	"""Version parser class."""

	_regex = re.compile(VERSION_PATTERN, re.VERBOSE | re.IGNORECASE)

	@classmethod
	def get_software_version(cls, sw: SoftwareCommand) -> Version:
		"""
		Get parsed software version by calling version command and parsing output.

		Parameters
		----------
		sw: SoftwareCommand
			Software to get version of

		Raises
		------
		CommandException:
			Raised if version command fails
		VersionParseException:
			Raised if version string cannot be parsed

		Returns
		-------
		Version:
			Parsed pypa/packaging version
		"""
		result = Command.execute(f'{sw} --version')
		if result.returncode != 0:
			raise CommandException(f'Failed to retrieve {sw} version: ' + result.stderr)
		return cls.get_from_str(result.stdout)

	@classmethod
	def get_from_str(cls, version_string: str) -> Version:
		"""
		Get parsed version object from string.

		Parameters
		----------
		version_string: str
			Input version string

		Raises
		------
		VersionParseException:
			Raised if version string cannot be parsed

		Returns
		-------
		Version:
			Parsed pypa/packaging version
		"""
		match = cls._regex.search(version_string)
		if not match:
			raise VersionParseException(f'Invalid version string: {version_string}')
		return version.parse(version_string)
